/*
	PortForwarder - that do port forwarding and reverse connection.
    Copyright (C) 2007  Shin Ryu(kurishin@naver.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
package org.codemaru;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.util.Iterator;

class ComThread extends Thread {
	static Log log = ConsoleLog.getInstance();
	SocketChannel fromSocketChannel;
	SocketChannel toSocketChannel;

	public ComThread(SocketChannel fromSocketChannel, SocketChannel toSocketChannel) {
		this.fromSocketChannel = fromSocketChannel;
		this.toSocketChannel = toSocketChannel;
	}
	
	public void run() {		
		int ret;
		Iterator it;
		SelectionKey key;
		ByteBuffer fromBuf = null;
		ByteBuffer toBuf = null;
		Selector selector = null;
		
		fromBuf = ByteBuffer.allocateDirect(1024);
		toBuf = ByteBuffer.allocateDirect(1024);

		try {
			log.debug("begin...");
			selector = Selector.open();
			fromSocketChannel.configureBlocking(false);
			toSocketChannel.configureBlocking(false);
			fromSocketChannel.register(selector, SelectionKey.OP_READ, "F");
			toSocketChannel.register(selector, SelectionKey.OP_READ, "T");

			for (;;) {
				log.debug("selecting...");
				ret = selector.select();

				if (ret == 0) {
					log.debug("selector.select() == 0");
					continue;
				}
				
				log.debug("ret: "  + ret);
			
				it = selector.selectedKeys().iterator();

				while (it.hasNext()) {
					key = (SelectionKey)it.next();

					if (key.isReadable()) {
						log.debug("Readable: " + (String)key.attachment());
						if ("F".equals(key.attachment())) {
							if (((SocketChannel)key.channel()).read(fromBuf) < 0) {
								log.info("fromClient closed: " + ((SocketChannel)key.channel()).toString());
								return;
							}
							
							fromBuf.flip();
							while (fromBuf.hasRemaining())
								toSocketChannel.write(fromBuf);
							fromBuf.clear();
						}
						else {
							if (((SocketChannel)key.channel()).read(toBuf) < 0) {
								log.info("toClient closed: " + ((SocketChannel)key.channel()).toString());
								return;
							}
							toBuf.flip();
							while (toBuf.hasRemaining())
								fromSocketChannel.write(toBuf);
							toBuf.clear();
						}					
					}
					
					it.remove();
				}
			}
		}
		catch (Exception e) {
			log.error(e.getMessage());
		}
		finally {
			if (selector != null) {
				try {
					selector.close();
				} catch (IOException ignored) {
				}
			}
			
			if (fromSocketChannel != null) {
				try {
					fromSocketChannel.close();
				} catch (IOException ignored) {
				}
			}
			if (toSocketChannel != null) {
				try {
					toSocketChannel.close();
				} catch (IOException ignored) {
				}
			}
		}
	}
}
