/*
	PortForwarder - that do port forwarding and reverse connection.
    Copyright (C) 2007  Shin Ryu(kurishin@naver.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
package org.codemaru;

import java.util.Vector;

import org.codemaru.util.Config;
import org.codemaru.util.StringUtil;

/*
 	server.host=localhost
	server.port=8000
	log.level=DEBUG
	service.list=telnet
	telnet.server.host=192.168.1.113
	telnet.server.port=23
	telnet.forwarding.port=23 
 */

final class ForwardAttr {
	String serverHost;
	int serverPort;
	int forwardingPort;	
	
	ForwardAttr(String serverHost, int serverPort, int forwardingPort) {
		this.serverHost = serverHost;
		this.serverPort = serverPort;
		this.forwardingPort = forwardingPort;
	}
	
	public String getServerHost() {
		return serverHost;
	}
	
	public int getServerPort() {
		return serverPort;
	}
	
	public int getForwardingPort() {
		return forwardingPort;
	}
}

public class ConfigFactory {
	static String version = "0.7a";
	static String author = "Programmed by Shin Ryu(kurishin@naver.com)";
	
	static ConfigFactory instance = null;
	Vector fAttrList = new Vector();
	String fServerHost;
	int fServerPort;
	int fLogLevel;
	boolean fKeepAlive;
	int fTimeout;
	
	public static String getVersion() {
		return version;
	}
	
	public static String getAuthor() {
		return author;
	}
	
	private ConfigFactory() {
		String keys;
		String sLogLevel;
		
		sLogLevel = Config.getString("log.level");
		
		if ("DEBUG".equals(sLogLevel))
			fLogLevel = 0;
		else if ("INFO".equals(sLogLevel))
			fLogLevel = 1;
		else if ("WARNING".equals(sLogLevel))
			fLogLevel = 2;
		else if ("ERROR".equals(sLogLevel))
			fLogLevel = 3;
		else if ("FATAL".equals(sLogLevel))
			fLogLevel = 4;
		else fLogLevel = 1;
		
		fKeepAlive = "true".equals(Config.getString("keepalive.enabled"));
		fTimeout = Config.getInt("keepalive.timeout");
		if (fTimeout <= 0) {
			fTimeout = 60*60;
		}
		
		fServerHost = Config.getString("server.host");
		fServerPort = Config.getInt("server.port");
		
		if (fServerPort == 0)
			fServerPort = 8080;
		
		keys = Config.getString("service.list");
		if (keys == null) {
			System.err.println("no service.list found");
			return;
		}
		
		addAttributes(keys);
	}
	
	public static ConfigFactory getInstance() {
		if (instance == null)
			instance = new ConfigFactory();
			
		return instance;
	}
	
	public String getServerHost() {
		return fServerHost;
	}
	
	public int getServerPort() {
		return fServerPort;
	}
	
	public String getServiceHost(int index) {
		if (fAttrList == null || fAttrList.size() <= index)
			return "localhost";
		
		return ((ForwardAttr)fAttrList.get(index)).getServerHost();
	}
	
	public int getServicePort(int index) {
		if (fAttrList == null || fAttrList.size() <= index)
			return 0;
		
		return ((ForwardAttr)fAttrList.get(index)).getServerPort();
	}
	
	public int getForwardingPort(int index) {
		if (fAttrList == null || fAttrList.size() <= index)
			return 0;
		
		return ((ForwardAttr)fAttrList.get(index)).getForwardingPort();
	}
	
	public int getPortCount() {
		return fAttrList.size();
	}
	
	public int getLogLevel() {
		return fLogLevel;
	}
	
	public boolean getKeepAlive() {
		return fKeepAlive;
	}
	
	public int getTimeout() {
		return fTimeout;
	}
	
	void addAttributes(final String keys) {
		String[] list = StringUtil.split(keys, ",");
		String serverIp;
		int serverPort, forwardingPort;

		if (list == null)
			return;

		for (int i=0; i<list.length; i++) {
			if ("".equals(list[i].trim()))
				continue;

			serverIp = Config.getString(list[i] + ".server.host");
			serverPort = Config.getInt(list[i] + ".server.port");

			if (serverPort == 0)
				continue;
			
			forwardingPort = Config.getInt(list[i] + ".forwarding.port");
			if (forwardingPort == 0)
				forwardingPort = serverPort;

			fAttrList.add(new ForwardAttr(serverIp, serverPort, forwardingPort));			
		}
	}
}
