/*
	PortForwarder - that do port forwarding and reverse connection.
    Copyright (C) 2007  Shin Ryu(kurishin@naver.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
package org.codemaru;

import java.io.PrintStream;

import org.codemaru.util.DateUtil;

public class ConsoleLog implements Log {
	static Log instance = null;
	int logLevel;
	
	private ConsoleLog() {
		logLevel = ConfigFactory.getInstance().getLogLevel();
	}
	
	final static int DEBUG   = 0;
	final static int INFO    = 1;
	final static int WARNING = 2;
	final static int ERROR   = 3;
	final static int FATAL   = 4;

	public static Log getInstance() {
		if (instance == null) {
			instance = new ConsoleLog();
		}
		
		return instance;
	}
	
	synchronized private void log(int type, String log) {
		if (type < logLevel)
			return;
		
		PrintStream out;
		
		out = type == 2 | type == 3 ?System.err:System.out;
		switch (type) {
		case DEBUG:	
			out.print("[D][");
			break;
		case INFO:	
			out.print("[I][");
			break;
		case WARNING:	
			out.print("[W][");
			break;
		case ERROR:	
			out.print("[E][");
			break;
		case FATAL:	
			out.print("[F][");
			break;
		}

		out.print(DateUtil.getDateFormat());
		out.print("] ");
		out.println(log);
	}

	public void debug(String log) {
		log(DEBUG, log);
	}

	public void error(String log) {
		log(ERROR, log);		
	}

	public void fatal(String log) {
		log(FATAL, log);		
	}

	public void info(String log) {
		log(INFO, log);
	}
}
