/*
	PortForwarder - that do port forwarding and reverse connection.
    Copyright (C) 2007  Shin Ryu(kurishin@naver.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
package org.codemaru;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.BindException;
import java.net.InetSocketAddress;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.ClosedSelectorException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Iterator;

public class ForwardThread extends Thread {
	static Log log = ConsoleLog.getInstance();
	static ConfigFactory conf = ConfigFactory.getInstance();
	SocketChannel fSocketChannel = null;
	static final String kThreadNotFound = "Specified thread not found!\n";
	static long id = 0;
//	private IModuleFeedbackChannel log = LogFeedback.getInstance();
	
	public ForwardThread(SocketChannel socketChannel) {
		this.fSocketChannel = socketChannel;
	}
	
	void doServer() throws IOException {
		String port;
		BufferedReader br = new BufferedReader(new InputStreamReader(fSocketChannel.socket().getInputStream()));
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(fSocketChannel.socket().getOutputStream()));
		Selector selector = null;
		Iterator iter;
		
		log.debug("waiting readLine...");
		port = br.readLine();
		log.debug("read port: " + port);
		
		ServerSocketChannel sSockChannel = null;
		try {
			try {
				sSockChannel = ServerSocketChannel.open();
				sSockChannel.socket().bind(new InetSocketAddress(Integer.parseInt(port)));
				sSockChannel.configureBlocking(false);
				selector = Selector.open();

				sSockChannel.register(selector, SelectionKey.OP_ACCEPT);
			} catch (BindException e) {
				log.error(e.getMessage());
				return;
			} catch (IOException e) {
				bw.write(e.getMessage());
				bw.write("\n");
				log.error(e.getMessage());
				return;
			} 
			
			bw.write("OK\n");
			bw.flush();
			log.debug("OK");
			
			SocketChannel fromSocketChannel = null;
			int count;
			
			boolean keepAlive = conf.getKeepAlive();
			long timeout = conf.getTimeout()*1000;
			
			for (;;) {
				try {
					count = selector.select(timeout);
					if (count == 0) {
						log.debug("timeout...");
						if (keepAlive) {
							bw.write("\n");
							bw.flush();
						}
						continue;
					}
					iter = selector.selectedKeys().iterator();
				    
				    while (iter.hasNext())
				    {
				    	SelectionKey key = (SelectionKey)iter.next();
				    	
				    	if (key.isAcceptable())
				    	{
				    		fromSocketChannel = ((ServerSocketChannel)key.channel()).accept();
				    		

				    		log.debug("accepted " + fromSocketChannel.toString());
				    		synchronized (PServer.socketChannel) {
				    			bw.write(id + "\n");				    			
				    			PServer.socketChannel.put(new Long(id++), fromSocketChannel);				    			
				    		}				    		
				    		bw.flush();
//				    		fSocketChannel.socket().getOutputStream().write("Melong\n".getBytes());
				    		
				    		log.debug("complete " + fSocketChannel.socket().toString());
				    	}
				     
				    	iter.remove();
				    }
				}
				catch (ClosedChannelException e) {					
					e.printStackTrace();
					return;
				}
				catch (ClosedSelectorException e) {
					log.info("PClient has closed");				
					return;				
				} catch (IOException e) {
					log.error(e.getMessage());
					return;
				}
	             
			}
		}
		finally {
			br.close();
			bw.close();
			
			if (sSockChannel != null)
				sSockChannel.close();
		}
	}
	
	public void run() {
		try {
	        try {
				doServer();
			} catch (IOException e) {
				log.error(e.getMessage());
			}
		} finally {			
			try {
				if (fSocketChannel != null) fSocketChannel.close();
			} catch (IOException e) {
//				e.printStackTrace();
			}
		}
	}
}