package org.codemaru.util;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * @author Shin Ryu
 *
 */
public final class DateUtil {
	final private static int MONTHS[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	
	public static Date sysdate() {
		return new Date();
	}
	
	public static String getDate() {
		return getSeparatedDate(new Date(), "");
	}
	
	public static String getDate(final Date date) {
		return getSeparatedDate(date, "");
	}
	
	public static String getSeparatedDate() {
		return getSeparatedDate(new Date(), "/");
	}
	
	public static String getSeparatedDate(final String p) {
		return getSeparatedDate(new Date(), p);
	}
	
	public static String getSeparatedDate(final Date date) {
		return getSeparatedDate(date, "/");
	}
	
	public static String getSeparatedDate(final Date date, String p) {
		if (p == null)
			p = "";
		
		return (new SimpleDateFormat("yyyy" + p + "MM" + p + "dd", Locale.KOREA)).format(date);
	}
	
	public static String getSeparatedDate(final String date, String p) {
		if ( date == null || date.length() < 8)
			return date;
		
		if (p == null)
			p = "";
			
		return date.substring(0, 4) + p + date.substring(4, 6) + p + date.substring(6, 8);
	}
	
	public static String getDateFormat() {
		return getDateFormat(new Date(), null);
	}
	
	public static String getDateFormat(final Date date) {
		return getDateFormat(date, null);
	}
	
	public static String getDateFormat(final String f) {
		return getDateFormat(new Date(), f);
	}
	
	public static String getDateFormat(Date date, String f) {
		if (f == null)
			f = "yyyy/MM/dd HH:mm:ss";
		
		return (new SimpleDateFormat(f, Locale.KOREA)).format(date);
	}

	public static String getTime() {
		return getSeparatedTime(new Date(), "");
	}
	
	public static String getTime(Date date) {
		return getSeparatedTime(date, "");
	}
	
	public static String getSeparatedTime() {
		return getSeparatedTime(new Date(), ":");
	}
	
	public static String getSeparatedTime(final String p) {
		return getSeparatedTime(new Date(), p);
	}
	
	public static String getSeparatedTime(final Date date) {
		return getSeparatedTime(date, "");
	}
	
	public static String getSeparatedTime(final Date date, String p) {
		if (p == null)
			p = "";
		
		return (new SimpleDateFormat("HH" + p + "mm" + p + "ss", Locale.KOREA)).format(date);
	}
	
	public static String getSeparatedTime(final String date, String p) {
		if ( date == null || date.length() < 6)
			return date;
		
		if (p == null)
			p = "";
		
		return date.substring(0, 2) + p + date.substring(2, 4) + p + date.substring(4, 6);
	}
	
	public static long dateToDigit(final String date) {		
		int year, month, day;
        long total;        
        
		year  = Integer.parseInt(date.substring(0, 4));
        month = Integer.parseInt(date.substring(4, 6));
        day   = Integer.parseInt(date.substring(6, 8));
        
        total = (year-1)*365 + (year-1)/4 - (year-1)/100 + (year-1)/400;

        for (int i=0; i<month-1; i++) {
        	total += MONTHS[i];
        }
        
        if (month > 2 && (((year%4 == 0) && (year%100 != 0)) || (year%400 == 0)))
        	total++;
        
        total += day;
		return total;
	}
	
	public static String digitToDate(int year, int month, int day) {
		StringBuffer sb = new StringBuffer();
		sb.append(Integer.toString(year));
		if (month < 10)
			sb.append("0");
		sb.append(Integer.toString(month));
		if (day < 10)
			sb.append("0");
		sb.append(Integer.toString(day));
		
		return sb.toString();
//		return String.format("%04d%02d%02d", new Object[year, month, day]);
	}

	public static String digitToDate(long day) {
		int year, month = 1;
		long sum;
		boolean isYun;
		
 		sum = day/365 - 1;

		year = (int)(sum + (sum/4 - sum/100 + sum/400)/365);		

		day = day - (sum*365 + sum/4 - sum/100 + sum/400);
		isYun = ((year%4 == 0) && (year%100 != 0)) || (year%400 == 0);
		
		if (day <= 0) {
			year--;
			day += 365;
			if (isYun)
				day++;
		}
		
		for (int i=0; i<12; i++, month++) {	
			sum = MONTHS[i];
			
			if (isYun && i == 1)
				sum++;

			if (day - sum <= 0)
        		break;
			
        	day -= sum;
        }
		

		return digitToDate(year, month, (int)(day));
	}

	public static long dateDiff(final String date1, final String date2) {
		return dateToDigit(date2) - dateToDigit(date1);
	}

	public static String addDate(final String date, int n) {
        long total;

        total = dateToDigit(date);
        total += n;

        return digitToDate(total);
    }

	public static String addMonths(final String date, int n) {
        int year, month, day;

        if (date.length() != 8)
            return date;

        year  = Integer.parseInt(date.substring(0, 4));
        month = Integer.parseInt(date.substring(4, 6));
        day   = Integer.parseInt(date.substring(6, 8));

        year += n/12;
        month += n%12;

        if (month < 1) {
            year--;
            month += 12;
        }
        else if (month > 12) {
            year++;
            month -= 12;
        }

        if (day > 28) {
            switch (month) {
            case 2:
                boolean isYun = (year%4 == 0) && (year%100 != 0) || (year%400 == 0);
                day = isYun ? 29 : 28;
                break;
            case 4:
            case 6:
            case 9:
            case 11:
                if (day > 30)
                    day = 30;
                break;
            }
        }

		return digitToDate(year, month, day);
    }	
}
